# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
from dataiku.langchain.dku_llm import DKUChatLLM
import json

from langchain.output_parsers import PydanticOutputParser
from langchain.output_parsers import OutputFixingParser
from pydantic import BaseModel
from typing import List

df = dataiku.Dataset("extracted_json").get_dataframe()

LLM_ID = dataiku.get_custom_variables()["LLM_id"]
llm = DKUChatLLM(
    llm_id=LLM_ID,
    temperature=0
)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# Define the expected format through a Pydantic model
class Answer(BaseModel):
    product: str
    summary: str
    sentiment: str
    advantages: List[str]
    drawbacks: List[str]

# Define an OutputFixingParser, ie. a parser that leverages an LLM to correct errors if needed
parser = OutputFixingParser.from_llm(
    parser=PydanticOutputParser(pydantic_object=Answer),
    llm=llm
)

for i in range(len(df)):
    try:
        parsed = json.loads(df.iloc[i].prediction)
    except Exception as e:
        df.loc[i, "error"] = e
    parsed = parser.parse(df.iloc[i].prediction).dict()
    try:
        for k in Answer.schema()["properties"]:
            df.loc[i, f"extracted_{k}"] = json.dumps(parsed[k])
    except json.JSONDecodeError:
        pass

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("corrected_json").write_with_schema(df)