# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import re

LLM_ID = dataiku.get_custom_variables()["LLM_id"]
llm = dataiku.api_client().get_default_project().get_llm(LLM_ID)

df = dataiku.Dataset("product_reviews").get_dataframe()

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# The LLM is prompted to return a relatively complex JSON string.
# The next recipe will correct the formatting errors of this recipe.

instructions = """The user will provide examples of product reviews. Extract the following pieces of information:
- product covered by the review
- summary of the review in less than 10 words
- sentiment of the review. Either 'positive', 'neutral' or 'negative'
- advantages of the product, according to the product review (if any)
- drawbacks of the product, according to the product review (if any)

The answer should be a JSON string with the following format `{"product": "[name of the product or 'unknown' if the product is unspecified]", "summary": "[summary in less than 10 words]", "sentiment": "[sentiment of the review. Either 'positive', 'neutral' or 'negative']", "advantages": "[string list of the product's advantages or [] if none mentioned in the review]", "drawbacks": "[string list of the product's drawbacks or [] if none mentioned in the review]"}`
"""
prompt = [{"role": "system", "content": instructions}]

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
def extract_between_curly_braces(s):
    """
    Extract the part of a string between the first left curly bracket and the last right curly bracket.
    """
    pattern = r'^[^\}]*(\{.*?\})[^\}]*$'
    match = re.search(pattern, s, re.DOTALL)
    if match:
        return match.group(1)
    else:
        return s

for i in range(len(df)):
    completion = llm.new_completion()
    completion.settings["temperature"] = 0
    completion.settings["maxOutputTokens"] = 200
    completion.with_message(instructions, role="assistant")
    result = completion.with_message(df.loc[i, "text"]).execute().text
    df.loc[i, "prediction"] = extract_between_curly_braces(result)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("extracted_json").write_with_schema(df)