# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import os
os.environ["HOME"] = os.getcwd() # So that LMQL is allowed to write in local folders

import dataiku
import lmql
import json
import re

# Provide the OpenAI API key
auth_info = dataiku.api_client().get_auth_info(with_secrets=True)
for secret in auth_info["secrets"]:
    if secret["key"] == "openai_key":
        os.environ["OPENAI_API_KEY"] = secret["value"]

df = dataiku.Dataset("product_reviews").get_dataframe()

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
LMQL_QUERY_TEMPLATE = '''argmax
"""Extract the following pieces of information from a product review: the type of the product ("unknown" if not provided; if the model or brand is included, be as specific as possible), the summary of the review in less than 10 words, the sentiment of the review, and the main advantages and the main drawbacks, if any, mentioned in the review. The answer should be a JSON string.
Product review: {review}
JSON Output: {{{{"""
'"product": "[PRODUCT]", '
'"sentiment": "[SENTIMENT]", '
'"summary": "[SUMMARY]", '
'"advantages": [[[FIRST]'
not_closed = (FIRST != "]")
while not_closed:
    '[FEATURE]"[LAST]'
    not_closed = LAST == ', "'
', "drawbacks": [[[FIRST]'
not_closed = (FIRST != "]")
while not_closed:
    '[FEATURE]"[LAST]'
    not_closed = LAST == ', "'
'}}}}'
from
"openai/gpt-3.5-turbo-instruct"
where
STOPS_BEFORE(PRODUCT, '"') and
SENTIMENT in ['positive', 'neutral', 'negative'] and
STOPS_BEFORE(SUMMARY, '"') and
FIRST in ['"', ']'] and
STOPS_BEFORE(FEATURE, '"') and
LAST in [', "', ']']
'''

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
for i in df.index:
    result = lmql.run_sync(
        LMQL_QUERY_TEMPLATE.format(
            review=re.sub(r'\s+', ' ', df.loc[i, "text"]).replace('"', "'")
        )
    )
    d = json.loads(result[0].prompt.split("JSON Output: ")[1])
    for k in d:
        if k == "sentiment":
            df.loc[i, "predicted_sentiment"] = d[k]
        elif k in ["advantages", "drawbacks"]:
            df.loc[i, k] = json.dumps(d[k])
        else:
            df.loc[i, k] = d[k]
        

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("extracted_json_lmql").write_with_schema(df)