# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import pandas as pd
import torch
from transformers import AutoTokenizer, AutoModelForSequenceClassification

BATCH_SIZE = 32
DEVICE = torch.device("cuda:0" if torch.cuda.is_available() else "cpu")
MODEL = 'unitary/toxic-bert'

df = dataiku.Dataset("movie_quotes").get_dataframe()

tokenizer = AutoTokenizer.from_pretrained(MODEL)
model = AutoModelForSequenceClassification.from_pretrained(MODEL).to(DEVICE).eval()

labels=('Toxic','Severe Toxic','Obscene/Sexual','Threat/Violent','Insulting','Identity Hate')

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
i = 0
results = None
while i < len(df):
    # Batch and tokenize the inputs
    batch = list(df.iloc[i:i+BATCH_SIZE].text)
    tokens = tokenizer(text=batch, return_tensors='pt', max_length=512, truncation=True, padding=True)
    
    # Predict the label of each input
    with torch.no_grad():
        result = model(tokens.input_ids.to(DEVICE)).logits
    if results is None:
        results = result
    else:
        results = torch.cat((results, result), axis=0)
    i += BATCH_SIZE

scores_df = pd.DataFrame(torch.sigmoid(results).detach().cpu().numpy(), columns=labels)
df = pd.concat([df, scores_df], axis=1)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("moderation_toxic-bert").write_with_schema(df)