# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
import re

LLM_ID = dataiku.get_custom_variables()["LLM_id"]
llm = dataiku.api_client().get_default_project().get_llm(LLM_ID)

df = dataiku.Dataset("po_predictions").get_dataframe()

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
PROMPT = """Please act as an impartial judge and evaluate the quality of the response provided by an
AI assistant to the user question displayed below. Your evaluation should consider factors
such as the helpfulness, relevance, accuracy, depth, clarity, and level of detail of
the response. Begin your evaluation by providing a short explanation. Be as objective as
possible. After providing your explanation, please rate the response on a scale of 0 to 100
by strictly following this format: "[[rating]]", for example: "Rating: [[83]]".

[Question]
{}

[Answer]
{}
"""

def extract_digit_from_string(input_string):
    """
    Extract the score provided by the LLM.
    """
    pattern = r'\[\[(\d*)\]\]'
    match = re.search(pattern, input_string)
    if match:
        digit = match.group(1)
        return int(digit)
    else:
        return -1

def evaluate(question, answer):
    """
    Evaluate the generated answer with a LLM.
    """
    prompt = PROMPT.format(question, answer)
    completion = llm.new_completion()
    completion.settings["temperature"] = 0
    completion.settings["maxOutputTokens"] = 512
    result = completion.with_message(prompt).execute().text
    return result, extract_digit_from_string(result)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
for i in df.index:
    question = df.at[i, "question"]
    answer_finetuned = df.at[i, "prediction_finetuned"]
    answer_base = df.at[i, "prediction_base"]

    df.at[i, "comment_finetuned"], df.at[i, "grade_finetuned"] = evaluate(question, answer_finetuned)
    df.at[i, "comment_base"], df.at[i, "grade_base"] = evaluate(question, answer_base)

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("po_predictions_assessed").write_with_schema(df)