# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
# -*- coding: utf-8 -*-
import dataiku
from openai import OpenAI
import tiktoken
import numpy as np

MODEL = "gpt-3.5-turbo-instruct"

enc = tiktoken.encoding_for_model(MODEL)

df = dataiku.Dataset("product_reviews").get_dataframe()

# Provide the OpenAI API key
auth_info = dataiku.api_client().get_auth_info(with_secrets=True)
for secret in auth_info["secrets"]:
    if secret["key"] == "openai_key":
        client = OpenAI(api_key=secret["value"])
        openai_api_key = secret["value"]
        break

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
prompt = "Decide whether a product review's sentiment is positive, neutral, or negative"

# We add few-shot examples to the prompt
examples = [
    (
        "I love my new chess board!",
        "positive"
    ),
    (
        "Not what I expected but I guess it'll do",
        "neutral"
    ),
    (
        "I'm so disappointed. The product seemed much better on the website",
        "negative"
    )
]
labels = set()
for text, sentiment in examples:
    prompt += f"\n\nProduct review:\n{text}\nSentiment:\n{sentiment}"
    labels.add(sentiment)
prompt += "\n\nProduct review:\n{}\nSentiment:\n"

# We adjust the logit biases so that only the allowed tokens can be generated
# The code below works only when the class labels are made of only 1 token
logit_bias = {}
for label in labels:
    tokens = enc.encode(label)
    for token in tokens:
        logit_bias[str(token)] = 100

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
for i in df.index:
    # Call the Text Completion endpoint of the OpenAI API
    result = client.completions.create(
        model=MODEL,
        prompt=prompt.format(df.at[i, "text"]),
        max_tokens=1,
        logit_bias=logit_bias,
        logprobs=1
    )

    df.at[i, "prediction"] = result.choices[0].text
    df.at[i, "score"] = np.exp(result.choices[0].logprobs.token_logprobs[0])

# -------------------------------------------------------------------------------- NOTEBOOK-CELL: CODE
dataiku.Dataset("sentiment_openai_completion").write_with_schema(df)